from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from . import signals
from .serializers import NotifySerializerV0, NotifySerializerV1


@method_decorator(csrf_exempt, name='dispatch')
class NotifyView(APIView):

    @staticmethod
    def success(message_type, message_data):
        signals.notification_received.send(
            sender=None,
            name=message_type,
            data=message_data
        )
        return Response(message_data)

    def post(self, request, format=None):
        serializer = NotifySerializerV1(data=request.data)
        if serializer.is_valid():
            signals.webhook_message_received.send(
                sender=None,
                type=serializer.data["type"],
                data=serializer.data["data"],
                id=serializer.data["id"],
                created_at=serializer.data["created_at"],
            )
            return Response(serializer.data)

        serializer_v0 = NotifySerializerV0(data=request.data)
        if serializer_v0.is_valid():
            signals.notification_received.send(
                sender=None,
                name=serializer_v0.data["name"],
                data=serializer_v0.data
            )
            return Response(serializer_v0.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



import jwt

from .serializers import PaymentSerializer, CancelOrderSerializer


class Talar:
    """
    Talar integration class. Used to perform actions for Talar API.
    """

    def __init__(self, project_id, key_id, key_secret,
                 base_url="https://secure.talar.app"):
        self.project_id = project_id

        self.key_id = key_id
        self.key_secret = key_secret

        self.base_url = base_url

    @property
    def url(self):
        return f'{self.base_url}/project/{self.project_id}/payment/create/'

    @property
    def cancel_order_url(self):
        return f'{self.base_url}/project/{self.project_id}/payment/cancel/'

    def _handle_payment_data(self, serializer, data: dict):
        talar_serializer = serializer(data=data)
        talar_serializer.is_valid(raise_exception=True)

        encrypted_data = jwt.encode(
            talar_serializer.data,
            self.key_secret,
            algorithm='HS256',
            headers={'kid': self.key_id}
        )
        return encrypted_data.decode("utf-8")

    def create_payment_data(self, data: dict):
        return self._handle_payment_data(PaymentSerializer, data)

    def cancel_payment_data(self, data: dict):
        return self._handle_payment_data(CancelOrderSerializer, data)



from django.dispatch import Signal

notification_received = Signal()  # webhook message v0
"""
Sent when received new webhook message.

Deprecated.

    Args:
        - sender: set to None
        - name: Webhook message type
        - data: Webhook message received payload
"""

webhook_message_received = Signal()  # webhook message v1
"""
Sent when received new webhook message

    Args:
        - sender: set to None
        - type: Webhook message type
        - data: Webhook message data
        - id: Webhook message id
        - created_at: Webhook message creation datetime
"""
